﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using IoCRelease.Entities;
using NUnit.Framework;

namespace IoCRelease.Tests
{
   [TestFixture]
   public class ReleaseDisposableEntities
   {
      [Test]
      public void TransientDisposableObjectFailure()
      {
         try
         {
            using (IIoC ioc = new CastleIoC())
            {
               for (Int32 I = 0; I < 10000; ++I)
               {
                  // ask the container to create a transient object
                  ITestEntity var = ioc.Resolve<ITestEntity>("DisposableEntity");
                  if (I % 100 == 0)
                  {
                     GC.Collect(2);

                     Console.WriteLine("Iteration: {0}, Memory {1} Dispose {2} Finalize {3}",
                                       I, GC.GetTotalMemory(false), DisposableEntity.NumOfDispose, DisposableEntity.NumOfFinalize);
                  }
                  // ioc.Release(var);
               }
            }
            Console.WriteLine("Number of Dispose Called {0}, Finalize {1}", DisposableEntity.NumOfDispose, DisposableEntity.NumOfFinalize);
         }
         catch (Exception ex)
         {
            Console.WriteLine("Out of Memory Exception, Dispose called after container destruction: {0}, Finalize: {1}", DisposableEntity.NumOfDispose, DisposableEntity.NumOfFinalize);
            throw;
         }
      }

      [Test]
      public void TotallyTransientDisposableObjectsLifestyleSuccess()
      {
         try
         {
            using (IIoC ioc = new CastleIoC())
            {
               for (Int32 I = 0; I < 10000; ++I)
               {
                  // ask the container to create a transient object
                  ITestEntity var = ioc.Resolve<ITestEntity>("DisposableEntityTransient");
                  if (I % 100 == 0)
                  {
                     GC.Collect(2);

                     Console.WriteLine("Iteration: {0}, Memory {1} Dispose {2} Finalize {3}",
                                       I, GC.GetTotalMemory(false), DisposableEntity.NumOfDispose, DisposableEntity.NumOfFinalize);
                  }
                  // ioc.Release(var);
               }
            }
            Console.WriteLine("Number of Dispose Called {0}, Finalize {1}", DisposableEntity.NumOfDispose, DisposableEntity.NumOfFinalize);
         }
         catch (Exception ex)
         {
            Console.WriteLine("Out of Memory Exception, Dispose called after container destruction: {0}, Finalize: {1}", DisposableEntity.NumOfDispose, DisposableEntity.NumOfFinalize);
            throw;
         }

      }
   }
}
